@@ -26,16 +26,20 @@
     THIS MODULE DOES NOT HAVE STABLE PUBLIC API
 """
 
-import ast
-from logging import getLogger
 from datetime import timedelta
+from logging import getLogger
+import ast
+import os
 
+from plainbox.i18n import gettext as _
 from plainbox.impl.commands import PlainBoxCommand
 from plainbox.impl.commands.checkbox import CheckBoxCommandMixIn
 from plainbox.impl.commands.checkbox import CheckBoxInvocationMixIn
-from plainbox.impl.session import SessionStateLegacyAPI as SessionState
 from plainbox.impl.resource import RequirementNodeVisitor
 from plainbox.impl.runner import JobRunner
+from plainbox.impl.session import SessionManager
+from plainbox.impl.session import SessionMetaData
+from plainbox.impl.session import SessionState
 
 
 logger = getLogger("plainbox.commands.special")
@@ -54,6 +58,10 @@ class AnalyzeInvocation(CheckBoxInvocationMixIn):
 
     def run(self):
         if self.ns.run_local:
+            if self.ns.print_desired_job_list:
+                self._print_desired_job_list()
+            if self.ns.print_run_list:
+                self._print_run_list()
             self._run_local_jobs()
         if self.ns.print_stats:
             self._print_general_stats()
@@ -67,25 +75,50 @@ class AnalyzeInvocation(CheckBoxInvocationMixIn):
             self._print_validation_report(self.ns.only_errors)
         if self.ns.print_requirement_report:
             self._print_requirement_report()
+        if self.ns.print_desired_job_list:
+            self._print_desired_job_list()
+        if self.ns.print_run_list:
+            self._print_run_list()
+
+    def _print_desired_job_list(self):
+        print(_("[Desired Job List]").center(80, '='))
+        for job in self.session.desired_job_list:
+            print("{}".format(job.id))
+
+    def _print_run_list(self):
+        print(_("[Run List]").center(80, '='))
+        for job in self.session.run_list:
+            print("{}".format(job.id))
 
     def _run_local_jobs(self):
-        print("[Running Local Jobs]".center(80, '='))
-        with self.session.open():
+        print(_("[Running Local Jobs]").center(80, '='))
+        manager = SessionManager.create_with_state(self.session)
+        try:
+            manager.state.metadata.title = "plainbox dev analyze session"
+            manager.state.metadata.flags = [SessionMetaData.FLAG_INCOMPLETE]
+            manager.checkpoint()
             runner = JobRunner(
-                self.session.session_dir, self.provider_list,
-                self.session.jobs_io_log_dir, command_io_delegate=self)
+                manager.storage.location, self.provider_list,
+                os.path.join(manager.storage.location, 'io-logs'),
+                command_io_delegate=self)
             again = True
             while again:
                 for job in self.session.run_list:
                     if job.plugin == 'local':
-                        if self.session.job_state_map[job.name].result.outcome is None:
-                            self._run_local_job(runner, job)
+                        if self.session.job_state_map[job.id].result.outcome is None:
+                            self._run_local_job(manager, runner, job)
                             break
                 else:
                     again = False
+            manager.state.metadata.flags = []
+            manager.checkpoint()
+        finally:
+            manager.destroy()
 
-    def _run_local_job(self, runner, job):
-        print("{job}".format(job=job.name))
+    def _run_local_job(self, manager, runner, job):
+        print("{job}".format(job=job.id))
+        manager.state.metadata.running_job_name = job.id
+        manager.checkpoint()
         result = runner.run_job(job)
         self.session.update_job_result(job, result)
         new_desired_job_list = self._get_matching_job_list(
@@ -93,62 +126,64 @@ class AnalyzeInvocation(CheckBoxInvocationMixIn):
         new_problem_list = self.session.update_desired_job_list(
             new_desired_job_list)
         if new_problem_list:
-            print("Problem list", new_problem_list)
+            print(_("Problem list"), new_problem_list)
             self.problem_list.extend(new_problem_list)
 
     def _print_general_stats(self):
-        print("[General Statistics]".center(80, '='))
-        print("Known jobs: {}".format(len(self.job_list)))
-        print("Selected jobs: {}".format(len(self.desired_job_list)))
+        print(_("[General Statistics]").center(80, '='))
+        print(_("Known jobs: {}").format(len(self.job_list)))
+        print(_("Selected jobs: {}").format(len(self.desired_job_list)))
 
     def _print_dependency_report(self):
-        print("[Dependency Report]".center(80, '='))
+        print(_("[Dependency Report]").center(80, '='))
         if self.problem_list:
             for problem in self.problem_list:
                 print(" * {}".format(problem))
         else:
-            print("Selected jobs have no dependency problems")
+            print(_("Selected jobs have no dependency problems"))
 
     def _print_interactivity_report(self):
-        print("[Interactivity Report]".center(80, '='))
+        print(_("[Interactivity Report]").center(80, '='))
         if not self.session.run_list:
             return
-        max_job_len = max(len(job.name) for job in self.session.run_list)
+        max_job_len = max(len(job.id) for job in self.session.run_list)
         fmt = "{{job:{}}} : {{interactive:11}} : {{duration}}".format(
             max_job_len)
         for job in self.session.run_list:
             print(
                 fmt.format(
-                    job=job.name,
+                    job=job.id,
                     interactive=(
-                        "automatic" if job.automated else "interactive"),
+                        _("automatic") if job.automated else _("interactive")),
                     duration=(
+                        # TODO: use python-babel to format localized timedelta
+                        # in 14.04+ as 12.04 babel API is too limited
                         timedelta(seconds=job.estimated_duration)
                         if job.estimated_duration is not None
-                        else "unknown")
+                        else _("unknown"))
                 )
             )
 
     def _print_estimated_duration_report(self):
-        print("[Estimated Duration Report]".center(80, '='))
-        print("Estimated test duration:")
+        print(_("[Estimated Duration Report]").center(80, '='))
+        print(_("Estimated test duration:"))
         automated, manual = self.session.get_estimated_duration()
-        print("   automated tests: {}".format(
+        print("   " + _("automated tests: {}").format(
             timedelta(seconds=automated) if automated is not None
-            else "cannot estimate"))
-        print("      manual tests: {}".format(
+            else _("cannot estimate")))
+        print("   " + _("manual tests: {}").format(
             timedelta(seconds=manual) if manual is not None
-            else "cannot estimate"))
-        print("             total: {}".format(
+            else _("cannot estimate")))
+        print("   " + _("total: {}").format(
             timedelta(seconds=manual + automated)
             if manual is not None and automated is not None
-            else "cannot estimate"))
+            else _("cannot estimate")))
 
     def _print_validation_report(self, only_errors):
-        print("[Validation Report]".center(80, '='))
+        print(_("[Validation Report]").center(80, '='))
         if not self.session.run_list:
             return
-        max_job_len = max(len(job.name) for job in self.session.run_list)
+        max_job_len = max(len(job.id) for job in self.session.run_list)
         fmt = "{{job:{}}} : {{problem}}".format(max_job_len)
         problem = None
         for job in self.session.run_list:
@@ -160,14 +195,14 @@ class AnalyzeInvocation(CheckBoxInvocationMixIn):
                 if only_errors:
                     continue
                 problem = ""
-            print(fmt.format(job=job.name, problem=problem))
+            print(fmt.format(job=job.id, problem=problem))
             if problem:
-                print("Job defined in {}".format(job.origin))
+                print(_("Job defined in {}").format(job.origin))
         if only_errors and problem is None:
-            print("No problems found")
+            print(_("No problems found"))
 
     def _print_requirement_report(self):
-        print("[Requirement Report]".center(80, '='))
+        print(_("[Requirement Report]").center(80, '='))
         if not self.session.run_list:
             return
         requirements = set()
@@ -178,7 +213,7 @@ class AnalyzeInvocation(CheckBoxInvocationMixIn):
                     for packages in [
                             resource.text for resource in
                             resource_program.expression_list
-                            if resource.resource_name == 'package']:
+                            if resource.resource_id == 'package']:
                         node = ast.parse(packages)
                         visitor = RequirementNodeVisitor()
                         visitor.visit(node)
@@ -200,38 +235,47 @@ class AnalyzeCommand(PlainBoxCommand, CheckBoxCommandMixIn):
 
     def register_parser(self, subparsers):
         parser = subparsers.add_parser(
-            "analyze", help="analyze how selected jobs would be executed")
+            "analyze", help=_("analyze how selected jobs would be executed"))
         group = parser.add_mutually_exclusive_group()
         group.add_argument(
             '-l', '--run-local',
             action='store_true', dest='run_local',
-            help='Run all selected local jobs, required to see true data')
+            help=_('run all selected local jobs, required to see true data'))
         group.add_argument(
             '-L', '--skip-local',
             action='store_false', dest='run_local',
-            help='Do not run local jobs')
+            # TRANSLATORS: please keep the word 'local' untranslated.
+            # It designates special type of jobs, not their location.
+            help=_('do not run local jobs'))
         group = parser.add_argument_group("reports")
         group.add_argument(
             '-s', '--print-stats', action='store_true',
-            help="Print general job statistics")
+            help=_("print general job statistics"))
         group.add_argument(
             "-d", "--print-dependency-report", action='store_true',
-            help="Print dependency report")
+            help=_("print dependency report"))
         group.add_argument(
             "-t", "--print-interactivity-report", action='store_true',
-            help="Print interactivity report")
+            help=_("print interactivity report"))
         group.add_argument(
             "-e", "--print-estimated-duration-report", action='store_true',
-            help="Print estimated duration report")
+            help=_("print estimated duration report"))
         group.add_argument(
             "-v", "--print-validation-report", action='store_true',
-            help="Print validation report")
+            help=_("print validation report"))
         group.add_argument(
             "-r", "--print-requirement-report", action='store_true',
-            help="Print requirement report")
+            help=_("print requirement report"))
         group.add_argument(
             "-E", "--only-errors", action='store_true', default=False,
-            help="When coupled with -v, only problematic jobs will be listed")
+            help=_(
+                "when coupled with -v, only problematic jobs will be listed"))
+        group.add_argument(
+            "-S", "--print-desired-job-list", action='store_true',
+            help=_("print desired job list"))
+        group.add_argument(
+            "-R", "--print-run-list", action='store_true',
+            help=_("print run list"))
         parser.set_defaults(command=self)
         # Call enhance_parser from CheckBoxCommandMixIn
         self.enhance_parser(parser)
