@@ -22,6 +22,7 @@
 ================================================
 """
 
+from collections import OrderedDict
 from concurrent.futures import ThreadPoolExecutor
 from io import BytesIO
 import logging
@@ -32,7 +33,10 @@ from plainbox.impl.exporter import get_all_exporters
 from plainbox.impl.runner import JobRunner
 from plainbox.impl.session import SessionStorageRepository
 from plainbox.impl.session.legacy import SessionStateLegacyAPI as SessionState
+from plainbox.impl.transport import get_all_transports
 
+from requests.exceptions import Timeout, ConnectionError
+from urllib.error import HTTPError
 
 logger = logging.getLogger("plainbox.highlevel")
 
@@ -42,7 +46,7 @@ class PlainBoxObject:
     A thin wrapper around some other plainbox object.
     """
 
-    def __init__(self, impl, name=None, group=None, children=None):
+    def __init__(self, impl, name=None, group=None, children=None, attrs=None):
         """
         Initialize a new PlainBoxObject with the specified internal
         implementation object and some meta-data.
@@ -55,6 +59,8 @@ class PlainBoxObject:
             Human-visible group (class) this object belongs to
         :param children:
             A list of children that this object has
+        :param attrs:
+            A list of attributes that this object has
         """
         self._impl = impl
         self._name = name
@@ -62,6 +68,9 @@ class PlainBoxObject:
             children = []
         self._children = children
         self._group = group
+        if attrs is None:
+            attrs = {}
+        self._attrs = attrs
 
     def __str__(self):
         """
@@ -108,6 +117,15 @@ class PlainBoxObject:
         """
         return self._children
 
+    @property
+    def attrs(self):
+        """
+        A mapping of key-value attributes that this object has
+
+        This mapping is mutable and is always guaranteed to exist.
+        """
+        return self._attrs
+
 
 # NOTE: This should merge with the service object below but I didn't want
 # to do it right away as that would have to alter Service.__init__() and
@@ -163,18 +181,59 @@ class Explorer:
             provider_obj = PlainBoxObject(
                 provider,
                 group="provider",
-                name=provider.name)
-            for job in provider.get_builtin_jobs():
+                name=provider.name,
+                attrs=OrderedDict((
+                    ('broken_i18n',
+                     provider.description == provider.tr_description()),
+                    ('name', provider.name),
+                    ('namespace', provider.namespace),
+                    ('version', provider.version),
+                    ('description', provider.description),
+                    ('tr_description', provider.tr_description()),
+                    ('jobs_dir', provider.jobs_dir),
+                    ('whitelists_dir', provider.whitelists_dir),
+                    ('data_dir', provider.data_dir),
+                    ('locale_dir', provider.locale_dir),
+                    ('gettext_domain', provider.gettext_domain),
+                )))
+            for job in provider.load_all_jobs()[0]:
                 job_obj = PlainBoxObject(
                     job,
                     group="job",
-                    name=job.name)
+                    name=job.id,
+                    attrs=OrderedDict((
+                        ('broken_i18n',
+                         job.summary == job.tr_summary()
+                         or job.description == job.tr_description()),
+                        ('id', job.id),
+                        ('partial_id', job.partial_id),
+                        ('summary', job.summary),
+                        ('tr_summary', job.tr_summary()),
+                        ('raw_summary', job.get_raw_record_value('summary')),
+                        ('description', job.description),
+                        ('raw_description',
+                         job.get_raw_record_value('description')),
+                        ('tr_description', job.tr_description()),
+                        ('plugin', job.plugin),
+                        ('command', job.command),
+                        ('user', job.user),
+                        ('environ', job.environ),
+                        ('estimated_duration', job.estimated_duration),
+                        ('depends', job.depends),
+                        ('requires', job.requires),
+                        ('origin', str(job.origin)),
+                    )))
                 provider_obj.children.append(job_obj)
             for whitelist in provider.get_builtin_whitelists():
                 whitelist_obj = PlainBoxObject(
                     whitelist,
                     group="whitelist",
-                    name=whitelist.name)
+                    name=whitelist.name,
+                    attrs=OrderedDict((
+                        ('name', whitelist.name),
+                        ('implicit_namespace', whitelist.implicit_namespace),
+                        ('origin', str(whitelist.origin)),
+                    )))
                 provider_obj.children.append(whitelist_obj)
             service_obj.children.append(provider_obj)
         # Milk each repository for session storage data
@@ -188,7 +247,11 @@ class Explorer:
                 storage_obj = PlainBoxObject(
                     storage,
                     group="storage",
-                    name=storage.location)
+                    name=storage.location,
+                    attrs=OrderedDict((
+                        ('location', storage.location),
+                        ('session_file', storage.session_file),
+                    )))
                 repo_obj.children.append(storage_obj)
         return service_obj
 
@@ -250,6 +313,24 @@ class Service:
         data_subset = exporter.get_session_data_subset(session)
         exporter.dump(data_subset, stream)
 
+    def get_all_transports(self):
+        return [transport for transport in get_all_transports()]
+
+    def send_data_via_transport(self, transport, where, options, data):
+        transport_cls = get_all_transports().get(transport)
+        if transport is None:
+            return "No transport with name '{}' was found".format(transport)
+        try:
+            transport = transport_cls(where, options)
+            json = transport.send(data)
+            return "Submission uploaded to: {}".format(json['url'])
+        except Timeout as error:
+            return "Request to {} timed out: {}".format(where, error)
+        except ConnectionError as error:
+            return "Unable to connect to {}: {}".format(where, error)
+        except HTTPError as error:
+            return "HTTP error"
+
     def prime_job(self, session, job):
         """
         Prime the specified job for running.
