@@ -31,6 +31,7 @@ from unittest import TestCase
 from plainbox import __version__ as version
 from plainbox.abc import IProvider1
 from plainbox.impl.box import main
+from plainbox.impl.clitools import ToolBase
 from plainbox.impl.commands.checkbox import CheckBoxInvocationMixIn
 from plainbox.impl.testing_utils import MockJobDefinition, suppress_warnings
 from plainbox.testing_utils.io import TestIO
@@ -170,7 +171,8 @@ class TestMain(TestCase):
             with self.assertRaises(SystemExit) as call:
                 main(['--version'])
             self.assertEqual(call.exception.args, (0,))
-        self.assertEqual(io.combined, "{}.{}.{}\n".format(*version[:3]))
+        self.assertEqual(io.combined, "{}\n".format(
+            ToolBase.format_version_tuple(version)))
 
     @suppress_warnings
     # Temporarily supress warnings (i.e. ResourceWarning) to work around
